using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Reflection;
using System.Web.Mvc;

namespace samples {

   /// <summary>
   /// Provides a set of static (Shared in Visual Basic) methods that simplify the creation
   /// of <see cref="SelectList"/> and <see cref="MultiSelectList"/> instances using the names 
   /// and values of enumeration types.
   /// </summary>
   public static class EnumList {

      /// <summary>
      /// Creates a <see cref="SelectList"/> instance using the
      /// names and values of the provided enumeration type.
      /// </summary>
      /// <param name="type">The enumeration type.</param>
      /// <returns>A <see cref="SelectList"/> instance.</returns>
      public static SelectList Create(Type type) {
         return Create(type, null);
      }

      /// <summary>
      /// Creates a <see cref="SelectList"/> instance using the
      /// names and values of the provided enumeration type.
      /// </summary>
      /// <param name="type">The enumeration type.</param>
      /// <param name="selectedValue">The selected value of the list.</param>
      /// <returns>A <see cref="SelectList"/> instance.</returns>
      public static SelectList Create(Type type, object selectedValue) {
         return Create(type, selectedValue, null);
      }

      /// <summary>
      /// Creates a <see cref="SelectList"/> instance using the
      /// names and values of the provided enumeration type.
      /// </summary>
      /// <param name="type">The enumeration type.</param>
      /// <param name="selectedValue">The selected value of the list.</param>
      /// <param name="valueFormat">A formatting string for the enumeration value (e.g. use "d" for a numeric value instead of text).</param>
      /// <returns>A <see cref="SelectList"/> instance.</returns>
      public static SelectList Create(Type type, object selectedValue, string valueFormat) {

         if (type == null) throw new ArgumentNullException("type");
         if (!type.IsEnum) throw new ArgumentException("type must be an Enum type.");

         List<SelectListItem> items = new List<SelectListItem>();

         foreach (object value in Enum.GetValues(type)) {

            string name = Enum.GetName(type, value);

            MemberInfo[] memInfo = type.GetMember(name);

            string itemText = GetDisplayName(memInfo[0]) ?? name;

            string itemValue = (!String.IsNullOrEmpty(valueFormat)) ?
               Enum.Format(type, value, valueFormat)
               : name;

            items.Add(
               new SelectListItem {
                  Text = itemText,
                  Value = itemValue
               }
            );
         }

         return new SelectList(items, "Value", "Text", selectedValue);
      }

      static string GetDisplayName(MemberInfo member) {

         DisplayAttribute displayAttr =
            Attribute.GetCustomAttribute(member, typeof(DisplayAttribute)) as DisplayAttribute;

         if (displayAttr != null)
            return displayAttr.GetName();

         DisplayNameAttribute displayNameAttr =
            Attribute.GetCustomAttribute(member, typeof(DisplayNameAttribute)) as DisplayNameAttribute;

         if (displayNameAttr != null)
            return displayNameAttr.DisplayName;

         return null;
      }

      /// <summary>
      /// Creates a <see cref="MultiSelectList"/> instance using the
      /// names and values of the provided enumeration type.
      /// </summary>
      /// <param name="type">The enumeration type.</param>
      /// <returns>A <see cref="MultiSelectList"/> instance.</returns>
      public static MultiSelectList CreateMulti(Type type) {
         return new MultiSelectList(Create(type), "Value", "Text");
      }

      /// <summary>
      /// Creates a <see cref="MultiSelectList"/> instance using the
      /// names and values of the provided enumeration type.
      /// </summary>
      /// <param name="type">The enumeration type.</param>
      /// <param name="selectedValues">The selected values of the list.</param>
      /// <returns>A <see cref="MultiSelectList"/> instance.</returns>
      public static MultiSelectList CreateMulti(Type type, IEnumerable selectedValues) {
         return new MultiSelectList(Create(type, null), "Value", "Text", selectedValues);
      }

      /// <summary>
      /// Creates a <see cref="MultiSelectList"/> instance using the
      /// names and values of the provided enumeration type.
      /// </summary>
      /// <param name="type">The enumeration type.</param>
      /// <param name="selectedValues">The selected values of the list.</param>
      /// <param name="valueFormat">A formatting string for the enumeration value (e.g. use "d" for a numeric value instead of text).</param>
      /// <returns>A <see cref="MultiSelectList"/> instance.</returns>
      public static MultiSelectList CreateMulti(Type type, IEnumerable selectedValues, string valueFormat) {
         return new MultiSelectList(Create(type, null, valueFormat), "Value", "Text", selectedValues);
      }
   }
}