﻿// Copyright 2011 Max Toro Q.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Collections.Specialized;
using System.Globalization;
using System.Runtime.Serialization;
using System.Text;
using System.Web;
using System.Web.Mvc;

namespace maxtoroq.web.mvc.html {

   /// <summary>
   /// Provides a set of static (Shared in Visual Basic) methods for rendering HTML 
   /// pagination links.
   /// </summary>
   public static class PagerExtensions {

      public static string ParameterViewDataKey { get; set; }
      public static string PageSizeViewDataKey { get; set; }
      public static string TotalResultsViewDataKey { get; set; }

      static PagerExtensions() {
         
         ParameterViewDataKey = null;
         PageSizeViewDataKey = "PageSize";
         TotalResultsViewDataKey = "Count";
      }

      public static MvcHtmlString Pager(this HtmlHelper htmlHelper) {
         return Pager(htmlHelper, null);
      }

      public static MvcHtmlString Pager(this HtmlHelper htmlHelper, PagerSettings settings) {

         ViewContext viewContext = htmlHelper.ViewContext;

         string paramViewDataKey = ParameterViewDataKey;

         if (paramViewDataKey == null) {
            
            if (settings == null)
               settings = new PagerSettings();

            paramViewDataKey = settings.ParameterName;
         }

         int startIndexOrPageNumber = (int)viewContext.ViewData[paramViewDataKey];
         int pageSize = (int)viewContext.ViewData[PageSizeViewDataKey];
         int totalResults = (int)viewContext.ViewData[TotalResultsViewDataKey];

         return Pager(htmlHelper, startIndexOrPageNumber, pageSize, totalResults, settings);
      }

      public static MvcHtmlString Pager(this HtmlHelper htmlHelper, int startIndexOrPageNumber, int pageSize, int totalResults) {
         return Pager(htmlHelper, startIndexOrPageNumber, pageSize, totalResults, null);
      }

      public static MvcHtmlString Pager(this HtmlHelper htmlHelper, int startIndexOrPageNumber, int pageSize, int totalResults, PagerSettings settings) {

         settings = (settings == null) ? 
            new PagerSettings() : settings.Clone();

         HttpRequestBase request = htmlHelper.ViewContext.RequestContext.HttpContext.Request;

         if (settings.UrlPath == null)
            settings.UrlPath = request.Path;

         if (settings.QueryString == null)
            settings.QueryString = new NameValueCollection(request.QueryString);

         PagerParameterType paramType = settings.ParameterType;

         int numberOfPages = (pageSize == 0) ? 
            0 : (int)Decimal.Ceiling(Decimal.Divide(totalResults, pageSize));

         int startIndex = (paramType == PagerParameterType.StartIndex) ?
            startIndexOrPageNumber
            : (startIndexOrPageNumber * pageSize) - pageSize;

         int currentPage = (numberOfPages == 0) ? 
            0 : (int)Decimal.Floor(Decimal.Divide(startIndex, pageSize) + 1);

         if (paramType == PagerParameterType.PageNumber) 
            System.Diagnostics.Debug.Assert(currentPage == startIndexOrPageNumber);

         int pagerCompressedDisplayLimit = (int)Decimal.Floor((settings.MaxPagesToDisplay / 2) - 1);
         
         int pagerStartPage = (currentPage <= pagerCompressedDisplayLimit) ? 
            1 : currentPage - pagerCompressedDisplayLimit;
         
         int pagerEndPage = ((currentPage + pagerCompressedDisplayLimit) >= numberOfPages) ? 
            numberOfPages : currentPage + pagerCompressedDisplayLimit;

         int displayingStart = startIndex + 1;
         
         int displayingEnd = (currentPage * pageSize) - ((numberOfPages == currentPage) ? 
            (startIndex + pageSize - totalResults) : 0);

         bool displayPrev = settings.AlwaysDisplayPreviousNext || currentPage > 1;
         bool displayNext = settings.AlwaysDisplayPreviousNext || currentPage != numberOfPages;

         var sb = new StringBuilder();
         
         if (displayPrev) {

            var prevBuilder = new TagBuilder("a") {
               Attributes = { 
                  { "class", "prev" },
                  { "title", settings.PreviousTitle }
               }
            };

            if (currentPage > 1) 
               prevBuilder.Attributes.Add("href", GetUrl(currentPage - 1, pageSize, settings));
            else
               prevBuilder.AddCssClass("nohref");

            prevBuilder.SetInnerText(settings.PreviousText);

            sb.Append(prevBuilder.ToString());
         }

         if (pagerStartPage > 1 
            && settings.AlwaysDisplayFirstLast) {

            var firstBuilder = new TagBuilder("a") {
               Attributes = { 
                  { "href", GetUrl(1, pageSize, settings) }
               }
            };

            firstBuilder.SetInnerText("1");

            sb.Append(firstBuilder.ToString());

            if ((pagerStartPage - 1) > 1) {

               var sep = new TagBuilder("a") {
                  Attributes = { 
                     { "class", "flsep" }
                  }
               };

               sep.SetInnerText(settings.FirstLastSeparator);

               sb.Append(sep.ToString());
            }
         }

         for (int p = pagerStartPage; p <= pagerEndPage; p++) { 

            var a = new TagBuilder("a");

            if (p == currentPage)
               a.Attributes.Add("class", "self");
            else
               a.Attributes.Add("href", GetUrl(p, pageSize, settings));

            a.SetInnerText(p.ToString(CultureInfo.InvariantCulture));

            sb.Append(a.ToString());
         }

         if ((pagerEndPage < numberOfPages) 
            && settings.AlwaysDisplayFirstLast) {

            if ((numberOfPages - pagerEndPage) > 1) {

               var sep = new TagBuilder("a") {
                  Attributes = { 
                     { "class", "flsep" }
                  }
               };

               sep.SetInnerText(settings.FirstLastSeparator);

               sb.Append(sep.ToString());
            }

            var lastBuilder = new TagBuilder("a") {
               Attributes = { 
                  { "href", GetUrl(numberOfPages, pageSize, settings) }
               }
            };

            lastBuilder.SetInnerText(numberOfPages.ToString(CultureInfo.InvariantCulture));

            sb.Append(lastBuilder.ToString());
         }

         if (displayNext) {

            var nextBuilder = new TagBuilder("a") {
               Attributes = { 
                  { "class", "next" },
                  { "title", settings.NextTitle }
               }
            };

            if (currentPage != numberOfPages) 
               nextBuilder.Attributes.Add("href", GetUrl(currentPage + 1, pageSize, settings));
            else
               nextBuilder.AddCssClass("nohref");

            nextBuilder.SetInnerText(settings.NextText);

            sb.Append(nextBuilder.ToString()); 
         }

         if (numberOfPages > 0 
            && settings.DisplaySummary
            && settings.SummaryFormatString != null) {

            var summaryBuilder = new TagBuilder("span") {
               Attributes = { 
                  { "class", "summary" }
               }
            };
            summaryBuilder.SetInnerText(String.Format(CultureInfo.InvariantCulture, settings.SummaryFormatString, displayingStart, displayingEnd, totalResults));

            sb.Append(summaryBuilder.ToString()); 
         }

         var pagerBuilder = new TagBuilder("div") {
            Attributes = { 
               { "class", settings.CssClass } 
            }
         };

         pagerBuilder.InnerHtml = sb.ToString();

         return MvcHtmlString.Create(pagerBuilder.ToString());
      }

      static string GetUrl(int pageNumber, int pageSize, PagerSettings settings) {

         PagerParameterType paramType = settings.ParameterType;
         Func<int, PagerSettings, string> urlCreator = settings.UrlCreator;

         int paramValue = (paramType == PagerParameterType.StartIndex) ?
            (pageNumber * pageSize) - pageSize 
            : pageNumber;

         if (urlCreator != null)
            return urlCreator(paramValue, settings);

         NameValueCollection query = settings.QueryString;

         bool includeParam = (paramType == PagerParameterType.StartIndex) ?
            paramValue > 0 : paramValue > 1;

         if (includeParam)
            query[settings.ParameterName] = paramValue.ToString(CultureInfo.InvariantCulture);
         else
            query.Remove(settings.ParameterName);

         return String.Concat(settings.UrlPath, ToQueryString(query, includeDelimiter: true));
      }

      static string ToQueryString(NameValueCollection qs, bool includeDelimiter) {

         StringBuilder sb = new StringBuilder();

         for (int i = 0; i < qs.AllKeys.Length; i++) {

            string key = qs.AllKeys[i];
            string[] values = qs.GetValues(key);

            if (values != null) {
               for (int j = 0; j < values.Length; j++) {

                  if (sb.Length > 0)
                     sb.Append('&');

                  sb.Append(HttpUtility.UrlEncode(key))
                     .Append('=')
                     .Append(HttpUtility.UrlEncode(values[j]));
               }
            }
         }

         if (includeDelimiter && sb.Length > 0)
            sb.Insert(0, '?');

         return sb.ToString();
      }
   }

   /// <summary>
   /// Class used to customize the behavior of <see cref="PagerExtensions"/>.
   /// </summary>
   public class PagerSettings {

      // IMPORTANT: When new properties are added Import and Reset must be updated.

      static readonly PagerSettings _Defaults = new PagerSettings(defaultsConstructor: true);

      public static PagerSettings Defaults { get { return _Defaults; } }

      public string UrlPath { get; set; }
      public NameValueCollection QueryString { get; set; }
      public string ParameterName { get; set; }
      public Func<int, PagerSettings, string> UrlCreator { get; set; }
      public PagerParameterType ParameterType { get; set; }

      public int MaxPagesToDisplay { get; set; }
      public string CssClass { get; set; }
      public string PreviousTitle { get; set; }
      public string PreviousText { get; set; }
      public string NextTitle { get; set; }
      public string NextText { get; set; }
      public string FirstLastSeparator { get; set; }
      public bool AlwaysDisplayPreviousNext { get; set; }
      public bool AlwaysDisplayFirstLast { get; set; }
      public bool DisplaySummary { get; set; }
      public string SummaryFormatString { get; set; }

      /// <summary>
      /// Initializes a new instance of the <see cref="PagerSettings"/> class,
      /// using the values from the <see cref="PagerSettings.Defaults"/> property.
      /// </summary>
      public PagerSettings() 
         : this(Defaults.ParameterType) { }

      /// <summary>
      /// Initializes a new instance of the <see cref="PagerSettings"/> class,
      /// using the values from the <see cref="PagerSettings.Defaults"/> property
      /// and the provided <paramref name="parameterType"/>.
      /// </summary>
      /// <param name="parameterType">The parameter type.</param>
      public PagerSettings(PagerParameterType parameterType) {

         Import(Defaults);

         this.ParameterType = parameterType;

         if (this.ParameterName == null) {
            this.ParameterName = (this.ParameterType == PagerParameterType.StartIndex) ?
               "start" : "page";
         }
      }

      private PagerSettings(bool defaultsConstructor) {
         Reset();
      }

      void Import(PagerSettings settings) {

         this.AlwaysDisplayFirstLast = settings.AlwaysDisplayFirstLast;
         this.AlwaysDisplayPreviousNext = settings.AlwaysDisplayPreviousNext;
         this.CssClass = settings.CssClass;
         this.DisplaySummary = settings.DisplaySummary;
         this.FirstLastSeparator = settings.FirstLastSeparator;
         this.MaxPagesToDisplay = settings.MaxPagesToDisplay;
         this.NextTitle = settings.NextTitle;
         this.NextText = settings.NextText;
         this.ParameterName = settings.ParameterName;
         this.ParameterType = settings.ParameterType;
         this.PreviousTitle = settings.PreviousTitle;
         this.PreviousText = settings.PreviousText;

         this.QueryString = settings.QueryString;

         if (this.QueryString != null)
            this.QueryString = new NameValueCollection(this.QueryString);

         this.SummaryFormatString = settings.SummaryFormatString;
         this.UrlCreator = settings.UrlCreator;
         this.UrlPath = settings.UrlPath;
      }

      /// <summary>
      /// Creates a copy of the current <see cref="PagerSettings"/>.
      /// </summary>
      /// <returns>A copy of the current <see cref="PagerSettings"/>.</returns>
      public PagerSettings Clone() {

         var newSettings = new PagerSettings();
         newSettings.Import(this);

         return newSettings;
      }

      /// <summary>
      /// Resets the members of the settings class to their original default values, that is,
      /// the values from the <see cref="PagerSettings.Defaults"/> property before any
      /// changes were made.
      /// </summary>
      public void Reset() {

         this.AlwaysDisplayFirstLast = true;
         this.AlwaysDisplayPreviousNext = false;
         this.CssClass = "pager";
         this.DisplaySummary = false;
         this.FirstLastSeparator = "…";
         this.MaxPagesToDisplay = 16;
         this.NextTitle = "Next";
         this.NextText = "Next ›";
         this.ParameterName = null;
         this.ParameterType = default(PagerParameterType);
         this.PreviousTitle = "Previous";
         this.PreviousText = "‹ Previous";
         this.QueryString = null;
         this.SummaryFormatString = "{0} - {1} of {2}";
         this.UrlCreator = null;
         this.UrlPath = null;
      }
   }

   /// <summary>
   /// Specifies the type of value that represents the start position in a paginated set.
   /// </summary>
   /// <seealso cref="PagerSettings"/>
   public enum PagerParameterType { 
      
      /// <summary>
      /// The parameter represents the zero-based start index.
      /// </summary>
      StartIndex,
      
      /// <summary>
      /// The parameter represents the one-based page number.
      /// </summary>
      PageNumber
   }
}
