﻿// Copyright 2011 Max Toro Q.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// RadioButtonListExtensions is based on SelectExtensions from ASP.NET MVC by 'Microsoft Corporation' 
// Copyright (c) Microsoft Corporation. All rights reserved.
// http://aspnetwebstack.codeplex.com/

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq.Expressions;
using System.Text;
using System.Web.Mvc;
using System.Web.Routing;

namespace maxtoroq.web.mvc.html {

   /// <summary>
   /// Provides a set of static (Shared in Visual Basic) methods for rendering sets of
   /// HTML radio buttons.
   /// </summary>
   public static class RadioButtonListExtensions {

      public static MvcHtmlString RadioButtonList(this HtmlHelper htmlHelper, string name) {
         return RadioButtonList(htmlHelper, name, null);
      }

      public static MvcHtmlString RadioButtonList(this HtmlHelper htmlHelper, string name, IEnumerable<SelectListItem> selectList) {
         return RadioButtonList(htmlHelper, name, selectList, (IDictionary<string, object>)null);
      }

      public static MvcHtmlString RadioButtonList(this HtmlHelper htmlHelper, string name, IEnumerable<SelectListItem> selectList, object htmlAttributes) {
         return RadioButtonList(htmlHelper, name, selectList, HtmlHelper.AnonymousObjectToHtmlAttributes(htmlAttributes));
      }

      public static MvcHtmlString RadioButtonList(this HtmlHelper htmlHelper, string name, IEnumerable<SelectListItem> selectList, IDictionary<string, object> htmlAttributes) {

         string fullHtmlFieldName = htmlHelper.ViewContext.ViewData.TemplateInfo.GetFullHtmlFieldName(name);

         if (String.IsNullOrEmpty(fullHtmlFieldName))
            throw new ArgumentException("name cannot be null or empty.", "name");

         bool selectListFromViewData = false;

         if (selectList == null) {
            selectList = htmlHelper.ViewData.Eval(fullHtmlFieldName) as IEnumerable<SelectListItem>;
            selectListFromViewData = true;
         }

         if (selectList == null)
            throw new ArgumentNullException("selectList");

         object value = null;
         ModelState state;

         if (htmlHelper.ViewData.ModelState.TryGetValue(fullHtmlFieldName, out state)
            && (state.Value != null)) {
            value = state.Value.ConvertTo(typeof(string), null);
         }

         if (!selectListFromViewData && (value == null))
            value = htmlHelper.ViewData.Eval(fullHtmlFieldName);

         if (value != null) {

            string strValue = Convert.ToString(value, CultureInfo.CurrentCulture);

            var list = new List<SelectListItem>();

            foreach (SelectListItem item in selectList) {
               item.Selected = String.Equals(strValue, item.Value ?? item.Text, StringComparison.OrdinalIgnoreCase);

               list.Add(item);
            }

            selectList = list;
         }

         var sb = new StringBuilder();

         htmlAttributes = (htmlAttributes == null) ?
            new RouteValueDictionary() : new RouteValueDictionary(htmlAttributes);

         htmlAttributes.Remove("checked");

         string currentGroup = null;
         bool groupOpen = false;

         foreach (SelectListItem item in selectList) {

            var groupedItem = item as GroupedSelectListItem;

            if (groupedItem != null) {

               if (groupedItem.GroupName != currentGroup) {

                  currentGroup = groupedItem.GroupName;

                  if (groupOpen) {
                     sb.Append("</div>");
                     groupOpen = false;
                  }

                  if (!String.IsNullOrEmpty(currentGroup)) {
                     
                     var groupLabel = new TagBuilder("label") {
                        Attributes = {
                           { "class", "option-group-label" }
                        },
                        InnerHtml = groupedItem.GroupName
                     };

                     sb.Append("<div class=\"option-group\">");
                     sb.Append(groupLabel.ToString());

                     groupOpen = true; 
                  }
               }

            } else {

               if (groupOpen) {
                  sb.Append("</div>");
                  groupOpen = false;
               }
            }

            var input = new TagBuilder("input");
            input.MergeAttributes(htmlAttributes);
            input.MergeAttribute("type", "radio");
            input.MergeAttribute("name", fullHtmlFieldName, replaceExisting: true);
            input.MergeAttribute("value", item.Value ?? item.Text ?? "", replaceExisting: true);

            if (item.Selected)
               input.MergeAttribute("checked", "checked");

            var label = new TagBuilder("label") { 
               InnerHtml = input.ToString(TagRenderMode.SelfClosing) + item.Text
            };

            sb.Append(label.ToString());
         }

         if (groupOpen) 
            sb.Append("</div>");

         return MvcHtmlString.Create(sb.ToString());
      }

      public static MvcHtmlString RadioButtonListFor<TModel, TProperty>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TProperty>> expression) {
         return RadioButtonListFor(htmlHelper, expression, null);
      }

      public static MvcHtmlString RadioButtonListFor<TModel, TProperty>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TProperty>> expression, IEnumerable<SelectListItem> selectList) {
         return RadioButtonListFor(htmlHelper, expression, selectList, (IDictionary<string, object>)null);
      }

      public static MvcHtmlString RadioButtonListFor<TModel, TProperty>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TProperty>> expression, IEnumerable<SelectListItem> selectList, object htmlAttributes) {
         return RadioButtonListFor(htmlHelper, expression, selectList, HtmlHelper.AnonymousObjectToHtmlAttributes(htmlAttributes));
      }

      public static MvcHtmlString RadioButtonListFor<TModel, TProperty>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TProperty>> expression, IEnumerable<SelectListItem> selectList, IDictionary<string, object> htmlAttributes) {

         if (expression == null) throw new ArgumentNullException("expression");

         return RadioButtonList(htmlHelper, ExpressionHelper.GetExpressionText(expression), selectList, htmlAttributes);
      }
   }
}
